{% extends 'spectator_events/base.html' %}
{% comment %}
Displays a single ClassicalWork or DancePiece.

As well as `object`, expects:
* breadcrumb_list_title - Text for linking to the parent list view.
* breadcrumb_list_url - URL for linking to the parent list view.
{% endcomment %}

{% block head_page_title %}{{ object.title }}{% endblock %}
{% block content_title %}{{ object.title }}{% endblock %}

{% block breadcrumbs %}
    {{ block.super }}
    <li class="breadcrumb-item"><a href="{{ breadcrumb_list_url }}">{{ breadcrumb_list_title }}</a></li>
    <li class="breadcrumb-item active">{{ object.title }}</li>
{% endblock %}

{% block content %}

    {% include 'spectator_core/includes/roles_list.html' with roles=object.roles.all heading='By' only %}

    {% include 'spectator_events/includes/visits_list.html' with events=object.event_set.all heading_level='h2' only %}

{% endblock content %}


{% block sidebar %}
    {% load spectator_core %}
    {% change_object_link_card object perms %}

    {{ block.super }}
{% endblock %}
