__title__ = 'Django Spectator'
__version__ = '5.2.0'
__author__ = 'Phil Gyford'
__author_email__ = 'phil@gyford.com'
__license__ = 'MIT'

VERSION = __version__  # synonym
TITLE = __title__  # synonym
